# (C) British Crown Copyright 2013 - 2014, Met Office
#
# This file is part of Iris.
#
# Iris is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Iris is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
"""
Test function :func:`iris.util.test_file_is_newer`.

"""

from __future__ import (absolute_import, division, print_function)

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

import os
import os.path
import shutil
import tempfile

from iris.util import file_is_newer_than


class TestFileIsNewer(tests.IrisTest):
    """Test the :func:`iris.util.file_is_newer_than` function."""

    def _name2path(self, filename):
        """Add the temporary dirpath to a filename to make a full path."""
        return os.path.join(self.temp_dir, filename)

    def setUp(self):
        # make a temporary directory with testfiles of known timestamp order.
        self.temp_dir = tempfile.mkdtemp('_testfiles_tempdir')
        # define the names of some files to create
        create_file_names = ['older_source_1', 'older_source_2',
                             'example_result',
                             'newer_source_1', 'newer_source_2']
        # create testfiles + ensure distinct 'mtime's in the required order.
        for i_file, file_name in enumerate(create_file_names):
            file_path = self._name2path(file_name)
            with open(file_path, 'w') as test_file:
                test_file.write('..content..')
            # Ensure 'mtime's are adequately separated and after create times.
            mtime = os.stat(file_path).st_mtime
            mtime += 5.0 + 10.0 * i_file
            os.utime(file_path, (mtime, mtime))

    def tearDown(self):
        # destroy whole contents of temporary directory
        shutil.rmtree(self.temp_dir)

    def _test(self, boolean_result, result_name, source_names):
        """Test expected result of executing with given args."""
        # Make args into full paths
        result_path = self._name2path(result_name)
        if isinstance(source_names, basestring):
            source_paths = self._name2path(source_names)
        else:
            source_paths = [self._name2path(name)
                            for name in source_names]
        # Check result is as expected.
        self.assertEqual(
            boolean_result,
            file_is_newer_than(result_path, source_paths))

    def test_no_sources(self):
        self._test(True, 'example_result', [])

    def test_string_ok(self):
        self._test(True, 'example_result', 'older_source_1')

    def test_string_fail(self):
        self._test(False, 'example_result', 'newer_source_1')

    def test_self_result(self):
        # This fails, because same-timestamp is *not* acceptable.
        self._test(False, 'example_result', 'example_result')

    def test_single_ok(self):
        self._test(True, 'example_result', ['older_source_2'])

    def test_single_fail(self):
        self._test(False, 'example_result', ['newer_source_2'])

    def test_multiple_ok(self):
        self._test(True, 'example_result', ['older_source_1',
                                            'older_source_2'])

    def test_multiple_fail(self):
        self._test(False, 'example_result', ['older_source_1',
                                             'older_source_2',
                                             'newer_source_1'])

    def test_wild_ok(self):
        self._test(True, 'example_result', ['older_sour*_*'])

    def test_wild_fail(self):
        self._test(False, 'example_result', ['older_sour*', 'newer_sour*'])

    def test_error_missing_result(self):
        with self.assertRaises(OSError) as error_trap:
            self._test(False, 'non_exist', ['older_sour*'])
        error = error_trap.exception
        self.assertEqual(error.strerror, 'No such file or directory')
        self.assertEqual(error.filename, self._name2path('non_exist'))

    def test_error_missing_source(self):
        with self.assertRaises(IOError) as error_trap:
            self._test(False, 'example_result', ['older_sour*', 'non_exist'])
        self.assertTrue(error_trap.exception.message.startswith(
            'One or more of the files specified did not exist'))

    def test_error_missing_wild(self):
        with self.assertRaises(IOError) as error_trap:
            self._test(False, 'example_result', ['older_sour*', 'unknown_*'])
        self.assertTrue(error_trap.exception.message.startswith(
            'One or more of the files specified did not exist'))


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2013 - 2014, Met Office
#
# This file is part of Iris.
#
# Iris is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Iris is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
"""Test function :func:`iris.util.new_axis`."""

from __future__ import (absolute_import, division, print_function)

# Import iris.tests first so that some things can be initialised before
# importing anything else.
import iris.tests as tests

import copy
import numpy as np
import unittest

import iris
from iris.util import new_axis


class Test(tests.IrisTest):
    def setUp(self):
        self.data = np.array([[1, 2], [1, 2]])
        self.cube = iris.cube.Cube(self.data)
        lat = iris.coords.DimCoord([1, 2], standard_name='latitude')
        lon = iris.coords.DimCoord([1, 2], standard_name='longitude')

        time = iris.coords.DimCoord([1], standard_name='time')
        wibble = iris.coords.AuxCoord([1], long_name='wibble')

        self.cube.add_dim_coord(lat, 0)
        self.cube.add_dim_coord(lon, 1)
        self.cube.add_aux_coord(time, None)
        self.cube.add_aux_coord(wibble, None)

        self.coords = {'lat': lat, 'lon': lon, 'time': time, 'wibble': wibble}

    def _assert_cube_notis(self, cube_a, cube_b):
        for coord_a, coord_b in zip(cube_a.coords(), cube_b.coords()):
            self.assertIsNot(coord_a, coord_b)

        self.assertIsNot(cube_a.metadata, cube_b.metadata)

        for factory_a, factory_b in zip(
                cube_a.aux_factories, cube_b.aux_factories):
            self.assertIsNot(factory_a, factory_b)

    def test_no_coord(self):
        # Providing no coordinate to promote.
        res = new_axis(self.cube)
        com = iris.cube.Cube(self.data[None])
        com.add_dim_coord(self.coords['lat'].copy(), 1)
        com.add_dim_coord(self.coords['lon'].copy(), 2)
        com.add_aux_coord(self.coords['time'].copy(), None)
        com.add_aux_coord(self.coords['wibble'].copy(), None)

        self.assertEqual(res, com)
        self._assert_cube_notis(res, self.cube)

    def test_scalar_dimcoord(self):
        # Providing a scalar coordinate to promote.
        res = new_axis(self.cube, 'time')
        com = iris.cube.Cube(self.data[None])
        com.add_dim_coord(self.coords['lat'].copy(), 1)
        com.add_dim_coord(self.coords['lon'].copy(), 2)
        com.add_aux_coord(self.coords['time'].copy(), 0)
        com.add_aux_coord(self.coords['wibble'].copy(), None)

        self.assertEqual(res, com)
        self._assert_cube_notis(res, self.cube)

    def test_scalar_auxcoord(self):
        # Providing a scalar coordinate to promote.
        res = new_axis(self.cube, 'wibble')
        com = iris.cube.Cube(self.data[None])
        com.add_dim_coord(self.coords['lat'].copy(), 1)
        com.add_dim_coord(self.coords['lon'].copy(), 2)
        com.add_aux_coord(self.coords['time'].copy(), None)
        com.add_aux_coord(self.coords['wibble'].copy(), 0)

        self.assertEqual(res, com)
        self._assert_cube_notis(res, self.cube)

    def test_maint_factory(self):
        # Ensure that aux factory persists.
        data = np.arange(12, dtype='i8').reshape((3, 4))

        orography = iris.coords.AuxCoord(
            [10, 25, 50, 5], standard_name='surface_altitude', units='m')

        model_level = iris.coords.AuxCoord(
            [2, 1, 0], standard_name='model_level_number')

        level_height = iris.coords.DimCoord(
            [100, 50, 10], long_name='level_height', units='m',
            attributes={'positive': 'up'},
            bounds=[[150, 75], [75, 20], [20, 0]])

        sigma = iris.coords.AuxCoord(
            [0.8, 0.9, 0.95], long_name='sigma',
            bounds=[[0.7, 0.85], [0.85, 0.97], [0.97, 1.0]])

        hybrid_height = iris.aux_factory.HybridHeightFactory(
            level_height, sigma, orography)

        cube = iris.cube.Cube(
            data, standard_name='air_temperature', units='K',
            dim_coords_and_dims=[(level_height, 0)],
            aux_coords_and_dims=[(orography, 1), (model_level, 0), (sigma, 0)],
            aux_factories=[hybrid_height])

        com = iris.cube.Cube(
            data[None], standard_name='air_temperature', units='K',
            dim_coords_and_dims=[(copy.copy(level_height), 1)],
            aux_coords_and_dims=[(copy.copy(orography), 2),
                                 (copy.copy(model_level), 1),
                                 (copy.copy(sigma), 1)],
            aux_factories=[copy.copy(hybrid_height)])
        res = new_axis(cube)

        self.assertEqual(res, com)
        self._assert_cube_notis(res, cube)


if __name__ == '__main__':
    unittest.main()



# (C) British Crown Copyright 2013 - 2014, Met Office
#
# This file is part of Iris.
#
# Iris is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Iris is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
"""Test function :func:`iris.util.rolling_window`."""

from __future__ import (absolute_import, division, print_function)

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

import numpy as np
import numpy.ma as ma

from iris.util import rolling_window


class Test_rolling_window(tests.IrisTest):

    def test_1d(self):
        # 1-d array input
        a = np.array([0, 1, 2, 3, 4], dtype=np.int32)
        expected_result = np.array([[0, 1], [1, 2], [2, 3], [3, 4]],
                                   dtype=np.int32)
        result = rolling_window(a, window=2)
        self.assertArrayEqual(result, expected_result)

    def test_2d(self):
        # 2-d array input
        a = np.array([[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]], dtype=np.int32)
        expected_result = np.array([[[0, 1, 2], [1, 2, 3], [2, 3, 4]],
                                    [[5, 6, 7], [6, 7, 8], [7, 8, 9]]],
                                   dtype=np.int32)
        result = rolling_window(a, window=3, axis=1)
        self.assertArrayEqual(result, expected_result)

    def test_1d_masked(self):
        # 1-d masked array input
        a = ma.array([0, 1, 2, 3, 4], mask=[0, 0, 1, 0, 0],
                     dtype=np.int32)
        expected_result = ma.array([[0, 1], [1, 2], [2, 3], [3, 4]],
                                   mask=[[0, 0], [0, 1], [1, 0], [0, 0]],
                                   dtype=np.int32)
        result = rolling_window(a, window=2)
        self.assertMaskedArrayEqual(result, expected_result)

    def test_2d_masked(self):
        # 2-d masked array input
        a = ma.array([[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]],
                     mask=[[0, 0, 1, 0, 0], [1, 0, 1, 0, 0]],
                     dtype=np.int32)
        expected_result = ma.array([[[0, 1, 2], [1, 2, 3], [2, 3, 4]],
                                    [[5, 6, 7], [6, 7, 8], [7, 8, 9]]],
                                   mask=[[[0, 0, 1], [0, 1, 0], [1, 0, 0]],
                                         [[1, 0, 1], [0, 1, 0], [1, 0, 0]]],
                                   dtype=np.int32)
        result = rolling_window(a, window=3, axis=1)
        self.assertMaskedArrayEqual(result, expected_result)

    def test_degenerate_mask(self):
        a = ma.array([[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]], dtype=np.int32)
        expected_result = ma.array([[[0, 1, 2], [1, 2, 3], [2, 3, 4]],
                                    [[5, 6, 7], [6, 7, 8], [7, 8, 9]]],
                                   mask=[[[0, 0, 0], [0, 0, 0], [0, 0, 0]],
                                         [[0, 0, 0], [0, 0, 0], [0, 0, 0]]],
                                   dtype=np.int32)
        result = rolling_window(a, window=3, axis=1)
        self.assertMaskedArrayEqual(result, expected_result)

    def test_step(self):
        # step should control how far apart consecutive windows are
        a = np.array([[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]], dtype=np.int32)
        expected_result = np.array([[[0, 1, 2], [2, 3, 4]],
                                    [[5, 6, 7], [7, 8, 9]]],
                                   dtype=np.int32)
        result = rolling_window(a, window=3, step=2, axis=1)
        self.assertArrayEqual(result, expected_result)

    def test_window_too_short(self):
        # raise an error if the window length is less than 1
        a = np.empty([5])
        with self.assertRaises(ValueError):
            rolling_window(a, window=0)

    def test_window_too_long(self):
        # raise an error if the window length is longer than the
        # corresponding array dimension
        a = np.empty([7, 5])
        with self.assertRaises(ValueError):
            rolling_window(a, window=6, axis=1)

    def test_invalid_step(self):
        # raise an error if the step between windows is less than 1
        a = np.empty([5])
        with self.assertRaises(ValueError):
            rolling_window(a, step=0)


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2014, Met Office
#
# This file is part of cf_units.
#
# cf_units is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cf_units is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with cf_units.  If not, see <http://www.gnu.org/licenses/>.
"""Unit tests for the `cf_units.Unit` class."""

from __future__ import (absolute_import, division, print_function)

import unittest

import numpy as np

import cf_units
from cf_units import Unit


class Test___init__(unittest.TestCase):

    def test_capitalised_calendar(self):
        calendar = 'GrEgoRian'
        expected = cf_units.CALENDAR_GREGORIAN
        u = Unit('hours since 1970-01-01 00:00:00', calendar=calendar)
        self.assertEqual(u.calendar, expected)

    def test_not_basestring_calendar(self):
        with self.assertRaises(TypeError):
            u = Unit('hours since 1970-01-01 00:00:00', calendar=5)


class Test_convert(unittest.TestCase):

    class MyStr(str):
        pass

    def test_gregorian_calendar_conversion_setup(self):
        # Reproduces a situation where a unit's gregorian calendar would not
        # match (using the `is` operator) to the literal string 'gregorian',
        # causing an `is not` test to return a false negative.
        cal_str = cf_units.CALENDAR_GREGORIAN
        calendar = self.MyStr(cal_str)
        self.assertIsNot(calendar, cal_str)
        u1 = Unit('hours since 1970-01-01 00:00:00', calendar=calendar)
        u2 = Unit('hours since 1969-11-30 00:00:00', calendar=calendar)
        u1point = np.array([8.], dtype=np.float32)
        expected = np.array([776.], dtype=np.float32)
        result = u1.convert(u1point, u2)
        return expected, result

    def test_gregorian_calendar_conversion_array(self):
        expected, result = self.test_gregorian_calendar_conversion_setup()
        np.testing.assert_array_equal(expected, result)

    def test_gregorian_calendar_conversion_dtype(self):
        expected, result = self.test_gregorian_calendar_conversion_setup()
        self.assertEqual(expected.dtype, result.dtype)

    def test_gregorian_calendar_conversion_shape(self):
        expected, result = self.test_gregorian_calendar_conversion_setup()
        self.assertEqual(expected.shape, result.shape)

    def test_non_gregorian_calendar_conversion_dtype(self):
        data = np.arange(4, dtype=np.float32)
        u1 = Unit('hours since 2000-01-01 00:00:00', calendar='360_day')
        u2 = Unit('hours since 2000-01-02 00:00:00', calendar='360_day')
        result = u1.convert(data, u2)
        self.assertEqual(result.dtype, np.float32)


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2014, Met Office
#
# This file is part of Iris.
#
# Iris is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Iris is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
"""Test function :func:`iris.util.array_equal`."""

from __future__ import (absolute_import, division, print_function)

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

import numpy as np
import numpy.ma as ma

from iris.util import array_equal


class Test(tests.IrisTest):
    def test_0d(self):
        array_a = np.array(23)
        array_b = np.array(23)
        array_c = np.array(7)
        self.assertTrue(array_equal(array_a, array_b))
        self.assertFalse(array_equal(array_a, array_c))

    def test_0d_and_scalar(self):
        array_a = np.array(23)
        self.assertTrue(array_equal(array_a, 23))
        self.assertFalse(array_equal(array_a, 45))

    def test_1d_and_sequences(self):
        for sequence_type in (list, tuple):
            seq_a = sequence_type([1, 2, 3])
            array_a = np.array(seq_a)
            self.assertTrue(array_equal(array_a, seq_a))
            self.assertFalse(array_equal(array_a, seq_a[:-1]))
            array_a[1] = 45
            self.assertFalse(array_equal(array_a, seq_a))

    def test_nd(self):
        array_a = np.array(np.arange(24).reshape(2, 3, 4))
        array_b = np.array(np.arange(24).reshape(2, 3, 4))
        array_c = np.array(np.arange(24).reshape(2, 3, 4))
        array_c[0, 1, 2] = 100
        self.assertTrue(array_equal(array_a, array_b))
        self.assertFalse(array_equal(array_a, array_c))

    def test_masked_is_ignored(self):
        array_a = ma.masked_array([1, 2, 3], mask=[1, 0, 1])
        array_b = ma.masked_array([2, 2, 2], mask=[1, 0, 1])
        self.assertFalse(array_equal(array_a, array_b))

    def test_fully_masked_arrays(self):
        array_a = ma.masked_array(np.arange(24).reshape(2, 3, 4), mask=True)
        array_b = ma.masked_array(np.arange(24).reshape(2, 3, 4), mask=True)
        self.assertTrue(array_equal(array_a, array_b))

    def test_fully_masked_0d_arrays(self):
        array_a = ma.masked_array(3, mask=True)
        array_b = ma.masked_array(3, mask=True)
        self.assertTrue(array_equal(array_a, array_b))

    def test_fully_masked_string_arrays(self):
        array_a = ma.masked_array(['a', 'b', 'c'], mask=True)
        array_b = ma.masked_array(['a', 'b', 'c'], mask=[1, 1, 1])
        self.assertTrue(array_equal(array_a, array_b))

    def test_partially_masked_string_arrays(self):
        array_a = ma.masked_array(['a', 'b', 'c'], mask=[1, 0, 1])
        array_b = ma.masked_array(['a', 'b', 'c'], mask=[1, 0, 1])
        self.assertTrue(array_equal(array_a, array_b))

    def test_string_arrays_equal(self):
        array_a = np.array(['abc', 'def', 'efg'])
        array_b = np.array(['abc', 'def', 'efg'])
        self.assertTrue(array_equal(array_a, array_b))

    def test_string_arrays_different_contents(self):
        array_a = np.array(['abc', 'def', 'efg'])
        array_b = np.array(['abc', 'de', 'efg'])
        self.assertFalse(array_equal(array_a, array_b))

    def test_string_arrays_subset(self):
        array_a = np.array(['abc', 'def', 'efg'])
        array_b = np.array(['abc', 'def'])
        self.assertFalse(array_equal(array_a, array_b))
        self.assertFalse(array_equal(array_b, array_a))

    def test_string_arrays_unequal_dimensionality(self):
        array_a = np.array('abc')
        array_b = np.array(['abc'])
        array_c = np.array([['abc']])
        self.assertFalse(array_equal(array_a, array_b))
        self.assertFalse(array_equal(array_b, array_a))
        self.assertFalse(array_equal(array_a, array_c))
        self.assertFalse(array_equal(array_b, array_c))

    def test_string_arrays_0d_and_scalar(self):
        array_a = np.array('foobar')
        self.assertTrue(array_equal(array_a, 'foobar'))
        self.assertFalse(array_equal(array_a, 'foo'))
        self.assertFalse(array_equal(array_a, 'foobar.'))


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2010 - 2014, Met Office
#
# This file is part of Iris.
#
# Iris is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Iris is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
"""Test function :func:`iris.util.describe_diff`."""

from __future__ import (absolute_import, division, print_function)

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

import numpy as np
import StringIO

import iris.cube
from iris.util import describe_diff


class Test(iris.tests.IrisTest):
    def setUp(self):
        self.cube_a = iris.cube.Cube([])
        self.cube_b = self.cube_a.copy()

    def _compare_result(self, cube_a, cube_b):
        result_strIO = StringIO.StringIO()
        describe_diff(cube_a, cube_b, output_file=result_strIO)
        return result_strIO.getvalue()

    def test_noncommon_array_attributes(self):
        # test non-common array attribute
        self.cube_a.attributes['test_array'] = np.array([1, 2, 3])
        return_str = self._compare_result(self.cube_a, self.cube_b)
        self.assertString(return_str, ['compatible_cubes.str.txt'])

    def test_same_array_attributes(self):
        # test matching array attribute
        self.cube_a.attributes['test_array'] = np.array([1, 2, 3])
        self.cube_b.attributes['test_array'] = np.array([1, 2, 3])
        return_str = self._compare_result(self.cube_a, self.cube_b)
        self.assertString(return_str, ['compatible_cubes.str.txt'])

    def test_different_array_attributes(self):
        # test non-matching array attribute
        self.cube_a.attributes['test_array'] = np.array([1, 2, 3])
        self.cube_b.attributes['test_array'] = np.array([1, 7, 3])
        return_str = self._compare_result(self.cube_a, self.cube_b)
        self.assertString(
            return_str,
            ['unit', 'util', 'describe_diff',
             'incompatible_array_attrs.str.txt'])


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2015, Met Office
#
# This file is part of cf_units.
#
# cf_units is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cf_units is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with cf_units.  If not, see <http://www.gnu.org/licenses/>.
"""Test function :func:`iris.unit.suppress_unit_warnings`."""

from __future__ import (absolute_import, division, print_function)

import warnings

import iris
from cf_units import suppress_unit_warnings


class Test(unittest.TestCase):

    def test_generic_warnings(self):
        unit_warning = ("Ignoring netCDF variable 'AL' invalid "
                        "units '(0 - 1)'")
        example_warning = 'Example warning'
        with warnings.catch_warnings(record=True) as filtered_warnings:
            with suppress_unit_warnings():
                warnings.warn(unit_warning)
                warnings.warn(example_warning)
        # Get to the actual warning strings for testing purposes.
        filtered_warnings_list = [w.message.message for w in filtered_warnings]
        self.assertNotIn(unit_warning, filtered_warnings_list)
        self.assertIn(example_warning, filtered_warnings_list)

    @tests.skip_data
    def test_load_file_with_captured_warnings(self):
        test_filename = tests.get_data_path(('NetCDF', 'testing', 'units.nc'))
        with warnings.catch_warnings(record=True) as filtered_warnings:
            with suppress_unit_warnings():
                iris.load(test_filename)
        filtered_warnings_list = [w.message.message for w in filtered_warnings]
        self.assertEqual(len(filtered_warnings_list), 0)


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2014, Met Office
#
# This file is part of Iris.
#
# Iris is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Iris is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Iris.  If not, see <http://www.gnu.org/licenses/>.
"""Test function :func:`iris.util.array_equal`."""

from __future__ import (absolute_import, division, print_function)

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

import copy

import numpy as np

import iris
import iris.tests.stock as stock
from iris.util import unify_time_units


class Test(tests.IrisTest):
    def simple_1d_time_cubes(self, calendar='gregorian'):
        coord_points = [1, 2, 3, 4, 5]
        data_points = [273, 275, 278, 277, 274]
        reftimes = ['hours since 1970-01-01 00:00:00',
                    'hours since 1970-01-02 00:00:00']
        list_of_cubes = []
        for reftime in reftimes:
            cube = iris.cube.Cube(np.array(data_points, dtype=np.float32),
                                  standard_name='air_temperature',
                                  units='K')
            unit = iris.unit.Unit(reftime, calendar=calendar)
            coord = iris.coords.DimCoord(points=np.array(coord_points,
                                                         dtype=np.float32),
                                         standard_name='time',
                                         units=unit)
            cube.add_dim_coord(coord, 0)
            list_of_cubes.append(cube)
        return list_of_cubes

    def _common(self, expected, result, coord_name='time'):
        # This tests time-like coords only.
        for cube in result:
            try:
                epoch = cube.coord(coord_name).units.origin
            except iris.exceptions.CoordinateNotFoundError:
                pass
            else:
                self.assertEqual(expected, epoch)

    def test_cubelist_with_time_coords(self):
        # Tests an :class:`iris.cube.CubeList` containing cubes with time
        # coords against a time string and a time coord.
        cubelist = iris.cube.CubeList(self.simple_1d_time_cubes())
        expected = 'hours since 1970-01-01 00:00:00'
        unify_time_units(cubelist)
        self._common(expected, cubelist)

    def test_list_of_cubes_with_time_coords(self):
        # Tests an iterable containing cubes with time coords against a time
        # string and a time coord.
        list_of_cubes = self.simple_1d_time_cubes()
        expected = 'hours since 1970-01-01 00:00:00'
        unify_time_units(list_of_cubes)
        self._common(expected, list_of_cubes)

    @tests.skip_data
    def test_no_time_coord_in_cubes(self):
        path0 = tests.get_data_path(('PP', 'aPPglob1', 'global.pp'))
        path1 = tests.get_data_path(('PP', 'aPPglob1', 'global_t_forecast.pp'))
        cube0 = iris.load_cube(path0)
        cube1 = iris.load_cube(path1)
        cubes = iris.cube.CubeList([cube0, cube1])
        result = copy.copy(cubes)
        unify_time_units(result)
        self.assertEqual(cubes, result)

    def test_time_coord_only_in_some_cubes(self):
        list_of_cubes = self.simple_1d_time_cubes()
        cube = stock.simple_2d()
        list_of_cubes.append(cube)
        expected = 'hours since 1970-01-01 00:00:00'
        unify_time_units(list_of_cubes)
        self._common(expected, list_of_cubes)

    def test_multiple_time_coords_in_cube(self):
        cube0, cube1 = self.simple_1d_time_cubes()
        units = iris.unit.Unit('days since 1980-05-02 00:00:00',
                               calendar='gregorian')
        aux_coord = iris.coords.AuxCoord(
            72, standard_name='forecast_reference_time', units=units)
        cube1.add_aux_coord(aux_coord)
        cubelist = iris.cube.CubeList([cube0, cube1])
        expected = 'hours since 1970-01-01 00:00:00'
        unify_time_units(cubelist)
        self._common(expected, cubelist)
        self._common(expected, cubelist, coord_name='forecast_reference_time')

    def test_multiple_calendars(self):
        cube0, cube1 = self.simple_1d_time_cubes()
        cube2, cube3 = self.simple_1d_time_cubes(calendar='360_day')
        cubelist = iris.cube.CubeList([cube0, cube1, cube2, cube3])
        expected = 'hours since 1970-01-01 00:00:00'
        unify_time_units(cubelist)
        self._common(expected, cubelist)


if __name__ == '__main__':
    tests.main()



# (C) British Crown Copyright 2010 - 2014, Met Office
#
# This file is part of cf_units.
#
# cf_units is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# cf_units is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with cf_units.  If not, see <http://www.gnu.org/licenses/>.
"""
Test Unit the wrapper class for Unidata udunits2.

"""

from __future__ import (absolute_import, division, print_function)

import unittest
import copy
import datetime as datetime
import operator

try:
    from operator import truediv
except ImportError:
    from operator import div as truediv

import numpy as np

from cf_units import cf_units as unit


Unit = unit.Unit


class TestCreation(unittest.TestCase):
    #
    # test: unit creation
    #
    def test_unit_fail_0(self):
        self.assertRaises(ValueError, Unit, 'wibble')

    def test_unit_pass_0(self):
        u = Unit('    meter')
        self.assertTrue(u.name, 'meter')

    def test_unit_pass_1(self):
        u = Unit('meter   ')
        self.assertTrue(u.name, 'meter')

    def test_unit_pass_2(self):
        u = Unit('   meter   ')
        self.assertTrue(u.name, 'meter')

    def test_calendar(self):
        calendar = unit.CALENDAR_365_DAY
        u = Unit('hours since 1970-01-01 00:00:00', calendar=calendar)
        self.assertEqual(u.calendar, calendar)

    def test_no_calendar(self):
        u = Unit('hours since 1970-01-01 00:00:00')
        self.assertEqual(u.calendar, unit.CALENDAR_GREGORIAN)

    def test_unknown_calendar(self):
        with self.assertRaises(ValueError):
            u = Unit('hours since 1970-01-01 00:00:00', calendar='wibble')


class TestModulus(unittest.TestCase):
    #
    # test: modulus property
    #
    def test_modulus_pass_0(self):
        u = Unit("degrees")
        self.assertEqual(u.modulus, 360.0)

    def test_modulus_pass_1(self):
        u = Unit("radians")
        self.assertEqual(u.modulus, np.pi*2)

    def test_modulus_pass_2(self):
        u = Unit("meter")
        self.assertEqual(u.modulus, None)


class TestConvertible(unittest.TestCase):
    #
    # test: convertible method
    #
    def test_convertible_fail_0(self):
        u = Unit("meter")
        v = Unit("newton")
        self.assertFalse(u.is_convertible(v))

    def test_convertible_pass_0(self):
        u = Unit("meter")
        v = Unit("mile")
        self.assertTrue(u.is_convertible(v))

    def test_convertible_fail_1(self):
        u = Unit('meter')
        v = Unit('unknown')
        self.assertFalse(u.is_convertible(v))
        self.assertFalse(v.is_convertible(u))

    def test_convertible_fail_2(self):
        u = Unit('meter')
        v = Unit('no unit')
        self.assertFalse(u.is_convertible(v))
        self.assertFalse(v.is_convertible(u))

    def test_convertible_fail_3(self):
        u = Unit('unknown')
        v = Unit('no unit')
        self.assertFalse(u.is_convertible(v))
        self.assertFalse(v.is_convertible(u))


class TestDimensionless(unittest.TestCase):
    #
    # test: dimensionless property
    #
    def test_dimensionless_fail_0(self):
        u = Unit("meter")
        self.assertFalse(u.is_dimensionless())

    def test_dimensionless_pass_0(self):
        u = Unit("1")
        self.assertTrue(u.is_dimensionless())

    def test_dimensionless_fail_1(self):
        u = Unit('unknown')
        self.assertFalse(u.is_dimensionless())

    def test_dimensionless_fail_2(self):
        u = Unit('no unit')
        self.assertFalse(u.is_dimensionless())


class TestFormat(unittest.TestCase):
    #
    # test: format method
    #
    def test_format_pass_0(self):
        u = Unit("watt")
        self.assertEqual(u.format(), "W")

    def test_format_pass_1(self):
        u = Unit("watt")
        self.assertEqual(u.format(unit.UT_ASCII), "W")

    def test_format_pass_2(self):
        u = Unit("watt")
        self.assertEqual(u.format(unit.UT_NAMES), "watt")

    def test_format_pass_3(self):
        u = Unit("watt")
        self.assertEqual(u.format(unit.UT_DEFINITION), "m2.kg.s-3")

    def test_format_pass_4(self):
        u = Unit('?')
        self.assertEqual(u.format(), 'unknown')

    def test_format_pass_5(self):
        u = Unit('nounit')
        self.assertEqual(u.format(), 'no_unit')


class TestName(unittest.TestCase):
    #
    # test: name property
    #
    def test_name_pass_0(self):
        u = Unit("newton")
        self.assertEqual(u.name, 'newton')

    def test_name_pass_1(self):
        u = Unit('unknown')
        self.assertEqual(u.name, 'unknown')

    def test_name_pass_2(self):
        u = Unit('no unit')
        self.assertEqual(u.name, 'no_unit')


class TestSymbol(unittest.TestCase):
    #
    # test: symbol property
    #
    def test_symbol_pass_0(self):
        u = Unit("joule")
        self.assertEqual(u.symbol, 'J')

    def test_symbol_pass_1(self):
        u = Unit('unknown')
        self.assertEqual(u.symbol, unit._UNKNOWN_UNIT_SYMBOL)

    def test_symbol_pass_2(self):
        u = Unit('no unit')
        self.assertEqual(u.symbol, unit._NO_UNIT_SYMBOL)


class TestDefinition(unittest.TestCase):
    #
    # test: definition property
    #
    def test_definition_pass_0(self):
        u = Unit("joule")
        self.assertEqual(u.definition, 'm2.kg.s-2')

    def test_definition_pass_1(self):
        u = Unit('unknown')
        self.assertEqual(u.definition, unit._UNKNOWN_UNIT_SYMBOL)

    def test_definition_pass_2(self):
        u = Unit('no unit')
        self.assertEqual(u.definition, unit._NO_UNIT_SYMBOL)


class TestOffset(unittest.TestCase):
    #
    # test: offset method
    #
    def test_offset_fail_0(self):
        u = Unit("meter")
        self.assertRaises(TypeError, operator.add, u, "naughty")

    def test_offset_fail_1(self):
        u = Unit('unknown')
        self.assertEqual(u + 10, 'unknown')

    def test_offset_fail_2(self):
        u = Unit('no unit')
        self.assertRaises(ValueError, operator.add, u, 10)

    def test_offset_pass_0(self):
        u = Unit("meter")
        self.assertEqual(u + 10, "m @ 10")

    def test_offset_pass_1(self):
        u = Unit("meter")
        self.assertEqual(u + 100.0, "m @ 100")

    def test_offset_pass_2(self):
        u = Unit("meter")
        self.assertEqual(u + 1000, "m @ 1000")


class TestOffsetByTime(unittest.TestCase):
    #
    # test: offset_by_time method
    #
    def test_offset_by_time_fail_0(self):
        u = Unit("hour")
        self.assertRaises(TypeError, u.offset_by_time, "naughty")

    def test_offset_by_time_fail_1(self):
        u = Unit("mile")
        self.assertRaises(ValueError, u.offset_by_time, 10)

    def test_offset_by_time_fail_2(self):
        u = Unit('unknown')
        self.assertRaises(ValueError, u.offset_by_time,
                          unit.encode_time(1970, 1, 1, 0, 0, 0))

    def test_offset_by_time_fail_3(self):
        u = Unit('no unit')
        self.assertRaises(ValueError, u.offset_by_time,
                          unit.encode_time(1970, 1, 1, 0, 0, 0))

    def test_offset_by_time_pass_0(self):
        u = Unit("hour")
        v = u.offset_by_time(unit.encode_time(2007, 1, 15, 12, 6, 0))
        self.assertEqual(v, "(3600 s) @ 20070115T120600.00000000 UTC")


class TestInvert(unittest.TestCase):
    #
    # test: invert method
    #
    def test_invert_fail_0(self):
        u = Unit('unknown')
        self.assertEqual(u.invert(), u)

    def test_invert_fail_1(self):
        u = Unit('no unit')
        self.assertRaises(ValueError, u.invert)

    def test_invert_pass_0(self):
        u = Unit("newton")
        self.assertEqual(u.invert(), "m-1.kg-1.s2")
        self.assertEqual(u.invert().invert(), "N")


class TestRoot(unittest.TestCase):
    #
    # test: root method
    #
    def test_root_fail_0(self):
        u = Unit("volt")
        self.assertRaises(TypeError, u.root, "naughty")

    def test_root_fail_1(self):
        u = Unit("volt")
        self.assertRaises(TypeError, u.root, 1.2)

    def test_root_fail_2(self):
        u = Unit("volt")
        self.assertRaises(ValueError, u.root, 2)

    def test_root_fail_3(self):
        u = Unit('unknown')
        self.assertEqual(u.root(2), u)

    def test_root_fail_4(self):
        u = Unit('no unit')
        self.assertRaises(ValueError, u.root, 2)

    def test_root_pass_0(self):
        u = Unit("volt^2")
        self.assertEqual(u.root(2), "V")


class TestLog(unittest.TestCase):
    #
    # test: log method
    #
    def test_log_fail_0(self):
        u = Unit("hPa")
        self.assertRaises(TypeError, u.log, "naughty")

    def test_log_fail_1(self):
        u = Unit('unknown')
        self.assertEqual(u.log(10), u)

    def test_log_fail_2(self):
        u = Unit('no unit')
        self.assertRaises(ValueError, u.log, 10)

    def test_log_pass_0(self):
        u = Unit("hPa")
        self.assertEqual(u.log(10), "lg(re 100 Pa)")


class TestMultiply(unittest.TestCase):
    def test_multiply_fail_0(self):
        u = Unit("amp")
        self.assertRaises(ValueError, operator.mul, u, "naughty")

    def test_multiply_fail_1(self):
        u = Unit('unknown')
        v = Unit('meters')
        self.assertTrue((u * v).is_unknown())
        self.assertTrue((v * u).is_unknown())

    def test_multiply_fail_3(self):
        u = Unit('unknown')
        v = Unit('no unit')
        self.assertRaises(ValueError, operator.mul, u, v)
        self.assertRaises(ValueError, operator.mul, v, u)

    def test_multiply_fail_5(self):
        u = Unit('meters')
        v = Unit('no unit')
        self.assertRaises(ValueError, operator.mul, u, v)
        self.assertRaises(ValueError, operator.mul, v, u)

    def test_multiply_pass_0(self):
        u = Unit("amp")
        self.assertEqual((u * 10).format(), "10 A")

    def test_multiply_pass_1(self):
        u = Unit("amp")
        self.assertEqual((u * 100.0).format(), "100 A")

    def test_multiply_pass_2(self):
        u = Unit("amp")
        self.assertEqual((u * 1000).format(), "1000 A")

    def test_multiply_pass_3(self):
        u = Unit("amp")
        v = Unit("volt")
        self.assertEqual((u * v).format(), "W")


class TestDivide(unittest.TestCase):
    def test_divide_fail_0(self):
        u = Unit("watts")
        self.assertRaises(ValueError, truediv, u, "naughty")

    def test_divide_fail_1(self):
        u = Unit('unknown')
        v = Unit('meters')
        self.assertTrue((u / v).is_unknown())
        self.assertTrue((v / u).is_unknown())

    def test_divide_fail_3(self):
        u = Unit('unknown')
        v = Unit('no unit')
        self.assertRaises(ValueError, truediv, u, v)
        self.assertRaises(ValueError, truediv, v, u)

    def test_divide_fail_5(self):
        u = Unit('meters')
        v = Unit('no unit')
        self.assertRaises(ValueError, truediv, u, v)
        self.assertRaises(ValueError, truediv, v, u)

    def test_divide_pass_0(self):
        u = Unit("watts")
        self.assertEqual((u / 10).format(), "0.1 W")

    def test_divide_pass_1(self):
        u = Unit("watts")
        self.assertEqual((u / 100.0).format(), "0.01 W")

    def test_divide_pass_2(self):
        u = Unit("watts")
        self.assertEqual((u / 1000).format(), "0.001 W")

    def test_divide_pass_3(self):
        u = Unit("watts")
        v = Unit("volts")
        self.assertEqual((u / v).format(), "A")


class TestPower(unittest.TestCase):
    def test_power(self):
        u = Unit("amp")
        self.assertRaises(TypeError, operator.pow, u, "naughty")
        self.assertRaises(TypeError, operator.pow, u, Unit('m'))
        self.assertRaises(TypeError, operator.pow, u, Unit('unknown'))
        self.assertRaises(TypeError, operator.pow, u, Unit('no unit'))
        self.assertEqual(u ** 2, Unit('A^2'))
        self.assertEqual(u ** 3.0, Unit('A^3'))
        self.assertEqual(u ** 4, Unit('A^4'))
        self.assertRaises(ValueError, operator.pow, u, 2.4)

        u = Unit("m^2")
        self.assertEqual(u ** 0.5, Unit('m'))
        self.assertRaises(ValueError, operator.pow, u, 0.4)

    def test_power_unknown(self):
        u = Unit('unknown')
        self.assertRaises(TypeError, operator.pow, u, "naughty")
        self.assertRaises(TypeError, operator.pow, u, Unit('m'))
        self.assertEqual(u ** 2, Unit('unknown'))
        self.assertEqual(u ** 3.0, Unit('unknown'))
        self.assertEqual(u ** 4, Unit('unknown'))

    def test_power_nounit(self):
        u = Unit('no unit')
        self.assertRaises(TypeError, operator.pow, u, "naughty")
        self.assertRaises(TypeError, operator.pow, u, Unit('m'))
        self.assertRaises(ValueError, operator.pow, u, 2)


class TestCopy(unittest.TestCase):
    #
    # test: copy method
    #
    def test_copy_pass_0(self):
        u = Unit("joule")
        self.assertEqual(copy.copy(u) == u, True)

    def test_copy_pass_1(self):
        u = Unit('unknown')
        self.assertTrue(copy.copy(u).is_unknown())

    def test_copy_pass_2(self):
        u = Unit('no unit')
        self.assertTrue(copy.copy(u).is_no_unit())


class TestStringify(unittest.TestCase):
    #
    # test: __str__ method
    #
    def test_str_pass_0(self):
        u = Unit("meter")
        self.assertEqual(str(u), "meter")

    #
    # test: __repr__ method
    #
    def test_repr_pass_0(self):
        u = Unit("meter")
        self.assertEqual(repr(u), "Unit('meter')")

    def test_repr_pass_1(self):
        u = Unit("hours since 2007-01-15 12:06:00",
                 calendar=unit.CALENDAR_STANDARD)
        comp = "Unit('hours since 2007-01-15 12:06:00', calendar='standard')"
        self.assertEqual(repr(u), comp)


class TestRichComparison(unittest.TestCase):
    #
    # test: __eq__ method
    #
    def test_eq_pass_0(self):
        u = Unit("meter")
        v = Unit("amp")
        self.assertEqual(u == v, False)

    def test_eq_pass_1(self):
        u = Unit("meter")
        v = Unit("m.s-1")
        w = Unit("hertz")
        self.assertEqual(u == (v / w), True)

    def test_eq_pass_2(self):
        u = Unit("meter")
        self.assertEqual(u == "meter", True)

    def test_eq_cross_category(self):
        m = Unit("meter")
        u = Unit('unknown')
        n = Unit('no_unit')
        self.assertFalse(m == u)
        self.assertFalse(m == n)
        self.assertFalse(u == n)

    #
    # test: __ne__ method
    #
    def test_neq_pass_0(self):
        u = Unit("meter")
        v = Unit("amp")
        self.assertEqual(u != v, True)

    def test_neq_pass_1(self):
        u = Unit("meter")
        self.assertEqual(u != 'meter', False)

    def test_ne_cross_category(self):
        m = Unit("meter")
        u = Unit('unknown')
        n = Unit('no_unit')
        self.assertTrue(m != u)
        self.assertTrue(m != n)
        self.assertTrue(u != n)


class TestOrdering(unittest.TestCase):
    def test_order(self):
        m = Unit("meter")
        u = Unit('unknown')
        n = Unit('no_unit')
        start = [m, u, n]
        self.assertEqual(sorted(start), [m, n, u])


class TestTimeEncoding(unittest.TestCase):
    #
    # test: encode_time module function
    #
    def test_encode_time_pass_0(self):
        result = unit.encode_time(2006, 1, 15, 12, 6, 0)
        self.assertEqual(result, 159019560.0)

    #
    # test: encode_date module function
    #
    def test_encode_date_pass_0(self):
        result = unit.encode_date(2006, 1, 15)
        self.assertEqual(result, 158976000.0)

    #
    # test: encode_clock module function
    #
    def test_encode_clock_pass_0(self):
        result = unit.encode_clock(12, 6, 0)
        self.assertEqual(result, 43560.0)

    #
    # test: decode_time module function
    #
    def test_decode_time_pass_0(self):
        result = unit.decode_time(158976000.0+43560.0)
        year, month, day, hour, min, sec, res = result
        self.assertEqual((year, month, day, hour, min, sec),
                         (2006, 1, 15, 12, 6, 0))


class TestConvert(unittest.TestCase):
    #
    # test: convert method
    #
    def test_convert_float_pass_0(self):
        u = Unit("meter")
        v = Unit("mile")
        self.assertEqual(u.convert(1609.344, v), 1.0)

    def test_convert_float_pass_1(self):
        u = Unit("meter")
        v = Unit("mile")
        a = (np.arange(2, dtype=np.float32) + 1) * 1609.344
        res = u.convert(a, v)
        e = np.arange(2, dtype=np.float32) + 1
        self.assertEqual(res[0], e[0])
        self.assertEqual(res[1], e[1])

    def test_convert_np_float(self):
        u = Unit("mile")
        v = Unit("meter")
        self.assertEqual(u.convert(np.float(1.0), v), 1609.344)
        self.assertEqual(u.convert(np.float16(1.0), v), 1609.344)
        self.assertEqual(u.convert(np.float32(1.0), v), 1609.344)
        self.assertEqual(u.convert(np.float64(1.0), v), 1609.344)

    def test_convert_double_pass_0(self):
        u = Unit("meter")
        v = Unit("mile")
        self.assertEqual(u.convert(1609.344, v, unit.FLOAT64), 1.0)

    def test_convert_double_pass_1(self):
        u = Unit("meter")
        v = Unit("mile")
        a = (np.arange(2, dtype=np.float64) + 1) * 1609.344
        res = u.convert(a, v, unit.FLOAT64)
        e = np.arange(2, dtype=np.float64) + 1
        self.assertEqual(res[0], e[0])
        self.assertEqual(res[1], e[1])

    def test_convert_int(self):
        u = Unit("mile")
        v = Unit("meter")
        self.assertEqual(u.convert(1, v), 1609.344)

    def test_convert_int_array(self):
        u = Unit("mile")
        v = Unit("meter")
        a = np.arange(2, dtype=np.int) + 1
        res = u.convert(a, v)
        e = (np.arange(2, dtype=np.float64) + 1) * 1609.344
        np.testing.assert_array_almost_equal(res, e, decimal=6)

    def test_convert_int_array_ctypearg(self):
        u = Unit("mile")
        v = Unit("meter")
        a = np.arange(2, dtype=np.int) + 1

        res = u.convert(a, v, unit.FLOAT32)
        e = (np.arange(2, dtype=np.float32) + 1) * 1609.344
        self.assertEqual(res.dtype, e.dtype)
        np.testing.assert_array_almost_equal(res, e, decimal=6)

        res = u.convert(a, v, unit.FLOAT64)
        e = (np.arange(2, dtype=np.float64) + 1) * 1609.344
        self.assertEqual(res.dtype, e.dtype)
        np.testing.assert_array_almost_equal(res, e, decimal=6)

    def test_convert_np_int(self):
        u = Unit("mile")
        v = Unit("meter")
        self.assertEqual(u.convert(np.int(1), v), 1609.344)
        self.assertEqual(u.convert(np.int8(1), v), 1609.344)
        self.assertEqual(u.convert(np.int16(1), v), 1609.344)
        self.assertEqual(u.convert(np.int32(1), v), 1609.344)
        self.assertEqual(u.convert(np.int64(1), v), 1609.344)

    def test_convert_fail_0(self):
        u = Unit('unknown')
        v = Unit('no unit')
        w = Unit('meters')
        x = Unit('kg')
        a = np.arange(10)

        # unknown and/or no-unit
        self.assertRaises(ValueError, u.convert, a, v)
        self.assertRaises(ValueError, v.convert, a, u)
        self.assertRaises(ValueError, w.convert, a, u)
        self.assertRaises(ValueError, w.convert, a, v)
        self.assertRaises(ValueError, u.convert, a, w)
        self.assertRaises(ValueError, v.convert, a, w)

        # Incompatible units
        self.assertRaises(ValueError, w.convert, a, x)

    def test_convert_time_ref_0(self):
        # Test converting from one reference time to another on non-standard
        # calendar.
        u1 = Unit('seconds since 1978-09-01 00:00:00', calendar='360_day')
        u2 = Unit('seconds since 1979-04-01 00:00:00', calendar='360_day')
        u1point = np.array([54432000.], dtype=np.float32)
        u2point = np.array([36288000.], dtype=np.float32)
        res = u1.convert(u1point, u2)
        np.testing.assert_array_almost_equal(res, u2point, decimal=6)

    def test_convert_time_fail_0(self):
        # Test converting from one reference time to another between different
        # calendars raises an error.
        u1 = Unit('seconds since 1978-09-01 00:00:00', calendar='360_day')
        u2 = Unit('seconds since 1979-04-01 00:00:00', calendar='gregorian')
        u1point = np.array([54432000.], dtype=np.float32)
        with self.assertRaises(ValueError):
            u1.convert(u1point, u2)


class TestNumsAndDates(unittest.TestCase):
    #
    # test: num2date method
    #
    def test_num2date_pass_0(self):
        u = Unit("hours since 2010-11-02 12:00:00",
                 calendar=unit.CALENDAR_STANDARD)
        self.assertEqual(str(u.num2date(1)), "2010-11-02 13:00:00")

    #
    # test: date2num method
    #
    def test_date2num_pass_0(self):
        u = Unit("hours since 2010-11-02 12:00:00",
                 calendar=unit.CALENDAR_STANDARD)
        d = datetime.datetime(2010, 11, 2, 13, 0, 0)
        self.assertEqual(str(u.num2date(u.date2num(d))), "2010-11-02 13:00:00")


class TestUnknown(unittest.TestCase):
    #
    # test: unknown units
    #
    def test_unknown_unit_pass_0(self):
        u = Unit("?")
        self.assertTrue(u.is_unknown())

    def test_unknown_unit_pass_1(self):
        u = Unit("???")
        self.assertTrue(u.is_unknown())

    def test_unknown_unit_pass_2(self):
        u = Unit("unknown")
        self.assertTrue(u.is_unknown())

    def test_unknown_unit_fail_0(self):
        u = Unit('no unit')
        self.assertFalse(u.is_unknown())

    def test_unknown_unit_fail_2(self):
        u = Unit('meters')
        self.assertFalse(u.is_unknown())


class TestNoUnit(unittest.TestCase):
    #
    # test: no unit
    #
    def test_no_unit_pass_0(self):
        u = Unit('no_unit')
        self.assertTrue(u.is_no_unit())

    def test_no_unit_pass_1(self):
        u = Unit('no unit')
        self.assertTrue(u.is_no_unit())

    def test_no_unit_pass_2(self):
        u = Unit('no-unit')
        self.assertTrue(u.is_no_unit())

    def test_no_unit_pass_3(self):
        u = Unit('nounit')
        self.assertTrue(u.is_no_unit())


class TestTimeReference(unittest.TestCase):
    #
    # test: time reference
    #
    def test_time_reference_pass_0(self):
        u = Unit('hours since epoch')
        self.assertTrue(u.is_time_reference())

    def test_time_reference_fail_0(self):
        u = Unit('hours')
        self.assertFalse(u.is_time_reference())


class TestTitle(unittest.TestCase):
    #
    # test: title
    #
    def test_title_pass_0(self):
        u = Unit('meter')
        self.assertEqual(u.title(10), '10 meter')

    def test_title_pass_1(self):
        u = Unit('hours since epoch', calendar=unit.CALENDAR_STANDARD)
        self.assertEqual(u.title(10), '1970-01-01 10:00:00')


class TestImmutable(unittest.TestCase):
    def _set_attr(self, unit, name):
        setattr(unit, name, -999)
        raise ValueError("'Unit' attribute '%s' is mutable!" % name)

    def test_immutable(self):
        u = Unit('m')
        for name in dir(u):
            self.assertRaises(AttributeError, self._set_attr, u, name)

    def test_hash(self):
        u1 = Unit('m')
        u2 = Unit('meter')
        u3 = copy.deepcopy(u1)
        h = set()
        for u in (u1, u2, u3):
            h.add(hash(u))
        self.assertEqual(len(h), 1)

        v1 = Unit('V')
        v2 = Unit('volt')
        for u in (v1, v2):
            h.add(hash(u))
        self.assertEqual(len(h), 2)


class TestInPlace(unittest.TestCase):

    def test1(self):
        # Check conversions do not change original object
        c = unit.Unit('deg_c')
        f = unit.Unit('deg_f')

        orig = np.arange(3, dtype=np.float32)
        converted = c.convert(orig, f)

        with self.assertRaises(AssertionError):
            np.testing.assert_array_equal(orig, converted)


if __name__ == '__main__':
    unittest.main()



import os
import sys
try:
    from setuptools import setup
    from setuptools.command.test import test as TestCommand
except ImportError:
    from distutils.core import setup


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['--verbose']
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


def extract_version():
    version = None
    fdir = os.path.dirname(__file__)
    fnme = os.path.join(fdir, 'cf_units', '__init__.py')
    with open(fnme) as fd:
        for line in fd:
            if (line.startswith('__version__')):
                _, version = line.split('=')
                # Remove quotation characters.
                version = version.strip()[1:-1]
                break
    return version


def file_walk_relative(top, remove=''):
    """
    Returns a generator of files from the top of the tree, removing
    the given prefix from the root/file result.

    """
    top = top.replace('/', os.path.sep)
    remove = remove.replace('/', os.path.sep)
    for root, dirs, files in os.walk(top):
        for file in files:
            yield os.path.join(root, file).replace(remove, '')

rootpath = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    return open(os.path.join(rootpath, *parts), 'r').read()


long_description = '{}'.format(read('README.rst'))

setup(
    name='cf_units',
    version=extract_version(),
    url='https://github.com/SciTools/cf_units',
    author='UK Met Office',
    packages=['cf_units', 'cf_units/tests'],
    package_data={'cf_units': list(file_walk_relative('cf_units/etc',
                                                      remove='cf_units/'))},
    data_files=[('cf_units', ['COPYING', 'COPYING.LESSER'])],
    tests_require=['pytest'],
    cmdclass={'test': PyTest},)
